//
// BAND LIMITED RMS MONITOR GENERATOR. Based on
// John Zweizigs DatTemplate.cc, found at
// http://www.ligo.caltech.edu/~jzweizig/DBase/DatTemplate.hh
//
//1/5/05 From BandMonitor_new.cc modification by KSG to 1. suppress filter
// ringdown and 2. to read a (circular buffer) 12 hour 
//archive file upon program restarts
//in order to avoid restarting with loss of all 12 hours of previous
//minute trends. 
#ifndef __CINT__

// ADD THIS WHEN YOU WANT TO GENERATE TRIGGERS. 

//-->  The next three lines are needed if you are going to generate triggers.
//     The descriptive title in PIDTITLE should the monitor function.
// #define PIDCVSHDR "$Header: /ldcg_server/common/repository_gds/gds/Monitors/blrms/BandMonitor.cc,v 1.16 2008/12/06 02:48:41 ahnitz Exp $"
// #define PIDTITLE  "Data monitor environment template"
// #include "ProcIdent.hh"

#include <time.h>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string>
#include "TSeries.hh"
#include "FSeries.hh"
#include "Dacc.hh"
#include "Interval.hh"
#include "Time.hh"
#include "DVector.hh"
#include "TSeries.hh"
#include "Trend.hh"

#include "IIRfilter_structures.hh"
#include "IIRfilter.hh"
#include "IIRfilter_bank.hh"
#include "IIRfilter_instance.hh"
#include "Rms_output.hh"
#include "Chan_filtset.hh"
#include "logicals.h"

#include "RmsConfig.hh"
#include "BandMonitor.hh"

#include "xsil/Xwriter.hh"
#include "xsil/xsil.hh"

//--------------------------------------  Generate the main routine.
EXECDAT(BandMonitor)
#endif               // !def(__CINT__)

using namespace std;

//--------------------------------------  Skeleton object constructor.
BandMonitor::BandMonitor(int argc, const char *argv[]) 
  : DatEnv(argc, argv), MaxFrame(999999), mLast_t0(0), mNconsecutive(1), mNconsecutive_last(0), minute_gap(false), minutes_since_gap(0), max_minutes_since(3), gap_size_t0(0), ring_down(false)  
{

  was_gap=0;

  read_count=0;
  // argument to hold parameter file name, output file name.
  //cout << "BandMonitor is at line 56 ksg" << endl;
  char input_param_file_name[ 100 ];
  sprintf(input_param_file_name,"default");
  char output_param_file_name[ 100 ];
  sprintf(output_param_file_name,"default");
  char update_param_file_name[ 100 ];
  sprintf(update_param_file_name,"none");
  char mon_server_name[ 100 ];
  sprintf(mon_server_name, "Band_Limited_RMs");
  total_seconds_to_run = 1000000000;
  // default to no trend writing
  make_trends = false;
  make_trend_index = false;
  // default trend name
  trend_name = "BLRMSDEF";
  // default to writing text files
  make_text_file = true;
  // KSG 8/26/05 addition of "DMT-BRMS_"after "H0:" etc in trend channel names
  trend_name_new = "DMT-BRMS_";
  // KSG 09/23/05 trend_name is now set equal to trend_name_new
  trend_name = trend_name_new;
  //cout << "BandMonitor is at line 72 ksg" << endl;
  //----------------------------------  Look for arguments
  //if((argc != 3) && ((argc != 5) && (argc != 7))) {
  if( ((argc%2)==0) || (argc < 3) ) {
    DumpHelp();
    exit(0);
  }
  for (int i=1 ; i<argc ; i++) {
    if (!strcmp("-if", argv[i])) {
      sprintf(input_param_file_name,"%s",argv[++i]);
    } else if (!strcmp("-of", argv[i])) {
      sprintf(output_param_file_name,"%s",argv[++i]);
    } else if (!strcmp("-uf", argv[i])) {
      sprintf(update_param_file_name,"%s",argv[++i]);
    } else if (!strcmp("-mons", argv[i])) {
      strcpy(mon_server_name, argv[++i]);
    } else if ( isDatEnvArg(argv[i]) ) {
      ++i;
    } else if (!strcmp("-mtxt", argv[i])) {
      if ( strcmp(argv[++i],"no") == 0 ) {
	make_text_file = false;
      }
    } else if (!strcmp("-midx", argv[i])) {
      if ( strcmp(argv[++i],"yes") == 0 ) {
	make_trend_index = true;
      }
    } else if (!strcmp("-trendname",argv[i])) {
      trend_name = argv[++i];
    } else {
      DumpHelp();
      finish();
      return;
    }
  }
  
  //----------------------------------  set the server name.
  setServerName(mon_server_name);

  //----------------------------------  set the time stride.
  float tStep(1.0);
  getDacc().setStride(tStep);
    
  //---------------------------------- get the channel name list
  
  rmspar = new RmsConfig( input_param_file_name ,
			  output_param_file_name ,
			  update_param_file_name );
  // rmspar->PrintChannelNames();
  
  // see if rms will be calculated once a minute. If yes, enable
  // generation of minute trends. If no, print a warning that no trends
  // will be generated. 
  if ( rmspar->secs_per_rms_reading == 60 ) {
    make_trends = true;
    pblrmsTrend = new Trend (trend_name.c_str(),Trend::kMinute);
  } else {
    cout << "Warning: rms refresh time (second last line of config" << endl
	 << "file) is not 60 (seconds), so no trends are created" << endl
	 << "by this monitor" << endl;
  }

  //---------------------------------- allocate memory for timeseries array
  ts = new TSeries* [ rmspar->number_of_channels ];
  tsaved = new TSeries* [rmspar->number_of_channels];

  //----------------------------------  Specify channels to scan
  for(int j=0; j < (rmspar->number_of_channels); ++j) {
    getDacc().addChannel( rmspar->channel_names[j] );
  }

  // set filter state to uninitialized
  are_filters_initialized = 0;
  // set data counter to zero.
  count_seconds = 0;
  // allocate memory for channel samp rates
  srate = new int [ rmspar->number_of_channels ];
  // allocate memory for filter instances
  filters = new Chan_filtset [ rmspar->number_of_channels ];
  number_of_filter_outputs = 0;
  segment_counter = 0;
  second_counter = 0;
  column_tally = 0;
  first_reading_done = false;
  // added by KSG on 11/05/04 to set minute_gap initially false
  //minute_gap = false;
  //cout << "BandMonitor is at line 150" << endl;
  // set number of rms measurements sent to dtt monitor output screen
  monitor_length = 720; // 720 minutes = 12 hours
  // set maximum length of the name for a monitor
  // KSG 09/23/05 The following value for the varible max_monitor_name_length is
  // increased from 40 to 50 to accomodate channel name changes
  //max_monitor_name_length = 40;
  max_monitor_name_length = 50;
  // the line below was uncommented by ksg and extra lines were added
   printf("Init is finished.\n");
   //cout << "BandMonitor is at line 163 ksg" << endl;
}

//--------------------------------------  Skeleton object destructor.
BandMonitor::~BandMonitor() 
{
   printf("Starting cleanup.\n");

  // the conditional on the cleanup determines if the
  // monitor ever started at all. If it did, we probably
  // need to delete the pointers allocated during startup.
  // This kluge will go away as soon as there is a decent
  // way of initializing quantities after reading one set
  // of data to get the data properties.
  if(are_filters_initialized != 0) {
    delete [] ts;
    delete [] tsaved;
    delete [] srate;
    delete file_output;
    delete [] filters;
    delete [] data_store;
    for(int i=0 ; i<column_tally ; ++i ) {
      delete monitor_tseries[i];
    }
    delete monitor_tseries;
    delete monitor_init;
    delete mon_name_buf;
    delete monitor_names;
    delete rmspar;
  }
  if( make_trends ) {
    delete pblrmsTrend;
  }
  //cout << "BandMonitor is at line 193 ksg" << endl;
}

void 
BandMonitor::DumpHelp(void)
{
  cout << "usage: -if <full path to configuration file and file name>" << endl
       << "        -of <full path to output data file and file name  >" << endl
       << "        -uf <file to contain only a single blrms reading >" << endl
       << "        -seccount <enter finite length (secs) before exit >" << endl
       << "        -mons <monitor server name (unique!!) >" << endl
       << "        -mtxt <no/other> disables text file output if no" << endl
       << "        -midx <yes/other> enables index for trends if yes" << endl
       << "        -trendname <text string (no spaces) for trend name >" 
       << endl;
  //cout << "at line 210 ksg 10/04" << endl;
  return;
}

void 
BandMonitor::Attention(void)
{
  MonServer::Attention();
}



//--------------------------------------  Frame processing function.
//-------------------------------------- do this step every timestride.
void
BandMonitor::ProcessData(void) {

/*
  read_count++;
  cout<<"Read Count="<<read_count<<"   "<<read_count%(1*60)<<endl;
  if (read_count%(1*60)==0){
	sleep(90);
        cout<<"sleeping"<<endl;
  }
*/
  

  for(int i=0; i < ( rmspar->number_of_channels ) ; ++i) {
    ts[i] = getDacc().refData( rmspar->channel_names[i] );
  }
  //cout << "BandMonitor is at line 227 ksg" << endl;
  //------- THIS CODE IS ONLY RUN THE FIRST TIME I TAKE DATA --------

  // Init filters the first time I take data
  if( are_filters_initialized == 0 ) {
    IIRfilter_bank iir_filters( rmspar->filter_filename );
    are_filters_initialized = 1;
    // Print to text file message that filters are initialized KSG 01/17/05
     cout << "IIR filters being initialized-line 244 Bandmonitor ksg" << endl;

    for(int k=0; k < rmspar->number_of_channels; ++k) {
      srate[k] = (*(ts+k))->getNSample();
      (filters+k)->fill_filters_for_channel( rmspar->channel_names[k],
					     srate[k], &iir_filters );
      number_of_filter_outputs += (filters+k)->no_of_iirfilt;
    
}
    file_output = new Rms_output ( rmspar->output_filename,
				   number_of_filter_outputs );

    // either or allow or inhibit writing of text files
    if( !make_text_file ) {
      file_output->disableTextOutput();
    }

    // write headers in output file
    // and use header information to name the trend channels
    // to be written to minute trend files. Minute trend files are
    // only created if the blrms update period is set to 60 seconds
    // in the blrms config file.

    int filter_count;
    int channel_count;
    char filter_name_buffer[30];
    int filter_srate;
    for(channel_count = 0; 
	channel_count < rmspar->number_of_channels;
	channel_count++ ) {
      for(filter_count = 0;
	  filter_count < ((filters+channel_count)->no_of_iirfilt);
	  filter_count++ ) {
	(filters + channel_count)->get_filter_name(filter_count,
						   filter_name_buffer);
	file_output->add_header( ((filters+channel_count)->channel_name),
				 filter_name_buffer,
				 column_tally );

	(filters + channel_count)->get_filter_srate(filter_count,
						    &filter_srate);
	file_output->update_samp_rate(column_tally,
				      filter_srate );
	//	printf("Column %d has sampling rate %d.\n", column_tally,
	//	       file_output->samp_rate_array[column_tally] );
	++column_tally;
      }
    }
    // allocate memory to array for storing mean square from data.
    data_store = new float [column_tally];

    // Now we know the number of channels, rmspar->number_of_channels,
    // the name of the kth channel, rmspar->channel_names[k],
    // the number of filters for the kth channel, (filters+k)->no_of_iirfilt,
    // the name of the jth filter for the kth data set, 
    // (filters+k)->getFilterName(char* filter_name, j), and
    // the total number of outputs, column_tally
    // So now we can allocate memory to the Timeseries to hold the
    // data to be sent to the monitor.


    // allocate memory to dtt update time series
    monitor_tseries = new TSeries* [ column_tally ];
    monitor_init = new float [ monitor_length ];
    for(int z=0;z<monitor_length;++z) {
      monitor_init[z] = 0;
    }

    Time start = ((*ts)->getStartTime() - 
		  double(monitor_length * rmspar->secs_per_rms_reading) *
		  (*ts)->getInterval());
    // cout << "Start time: " << start << endl;
		  
    for(int i=0 ; i<column_tally ; ++i) {
      monitor_tseries[i] = new TSeries( start,
	double(rmspar->secs_per_rms_reading) * (*ts)->getInterval(),
	(size_t) monitor_length ,
	monitor_init );
    }
    // fill array of names of monitor channels
    monitor_names = new char [ column_tally * 
			     max_monitor_name_length ];
    mon_name_buf = new char [max_monitor_name_length];
    int monitor_count(0);
    for(int i=0; i<rmspar->number_of_channels ; ++i) {
      for(int j=0; j<(filters+i)->no_of_iirfilt ; ++j) {
	(filters+i)->get_filter_name(j,mon_name_buf);
	sprintf(monitor_names + monitor_count*max_monitor_name_length,
		"%s_%s",
		(filters+i)->channel_name,
		mon_name_buf);

	// start data server
	// the following printf statement added by ksg on 10/21/04
	//printf("starting the data server ksg line 330");
	serveData(monitor_names+monitor_count*max_monitor_name_length,
	          monitor_tseries[monitor_count],0);

	// if trend writing is enabled, name channels after monitor names
	// KSG 8/24/05 the following 10 lines must be modified in order
	// To accomadate the new names for the channels with "DMT-BRMS_" in the middle
	if ( make_trends ) {
	  //KSG 09/23/05 the following definition of trendname as written 
	  //in the trend output .gwf file is being revised to accomadate the 
	  //new naming scheme for the DMT channels
	  string trendname(monitor_names + 
	  		   monitor_count*max_monitor_name_length);
	  // string trendname(monitor_names + trend_name_new + 
	  //		   monitor_count*max_monitor_name_length); this did not work
	  // KSG 9/17/05 the following cout statement added and then splitting of trendname in 2 parts called
	  //trendname1 and trendname2
	  string trendname1(trendname.substr(0,3));
	  string trendname2(trendname.substr(3,37));
	  // kludge: only looks at position 3... should check the whole string
	  if (trendname2[3] == '-') trendname2[3] = '_';
	  trendname = trendname1 + "DMT-BRMS_" + trendname2;
	  cout << "line 364 trendname1= "<< trendname1 << "trendname2= " << trendname2 << endl;
	  cout << "trendname = " << trendname << endl;
	  //cout << "before push_back trendname=" << trendname << endl;
	  trend_channel_list.push_back(trendname);
	  // KSG 9/17/05 the following cout statement added
	  //cout << " after push_back trendname=" << trendname << endl;
	  pblrmsTrend->addChannel(trendname.c_str());
	  cout << "adding channel " << trendname.c_str() << endl;
	}
	// end of code to name trend channels

	++monitor_count;
      }
    }
    //cout << "BandMonitor at line 341 ksg" << endl;
    // make index for trends
    if ( make_trends && make_trend_index ) {
      pblrmsTrend->writeIndex();
    }
    // now we are all ready to start writing time series of monitor data.

  }

  //------------ THIS CODE RUN ONCE EVERY ITERATION  ---------------


//cout<<"Reset Data 0th value: "<<data_store[8]<<endl;
  data_error = 0;
  column_count = 0;

if (!was_gap){
  for(int q = 0; q < rmspar->number_of_channels ; ++q) {
    // convert timeseries data to floats
    (*(ts+q))->Convert(DVector::t_float);
    // run filters on timeseries data
    if ( ((filters+q)->
	  run_filters( **(ts+q) , 
		       &column_count, 
		       data_store, NO ) ) == -1 ) {
      data_error = 1;
    }
    
  }
}
else{
IIRfilter_bank iir_filters( rmspar->filter_filename );
was_gap=0;
  //cout<<"Resetting filters"<<endl;
 for(int k=0; k < rmspar->number_of_channels; ++k) {
      //delete[] filters;
    //filters= new Chan_filtset [ rmspar->number_of_channels ];
      (filters+k)->fill_filters_for_channel( rmspar->channel_names[k],
					     srate[k], &iir_filters );
     for(int p=0;p<column_tally;++p) {
	data_store[p] = 0;
       if (p==0)cout<<"Reset Data Store: "<<data_store[0]<<endl;
      }
  }
//(filters+q)-> run_filters( **(tsaved+q) , &column_count, data_store, NO )
}


 //cout<<"Reset Data 0th value: "<<data_store[8]<<endl;
 
 //cout << "BandMonitor at line 366 ksg" << endl;
  if(data_error == 0) {
    if( segment_counter == 0 ) {
      tnow = (*ts)->getStartTime();
    }
    segment_counter++;
  }
  //cout << "BandMonitor at line 373 in once every iteration part ksg " << endl;  
  //------------ THIS SECTION RUN ONCE EVERY secs_per_rms_reading --
  // This segment inserted by KSG on 9/30/04 to check for time gaps in blrms 
  // time series; code was originally given to KSG by Keith Riles.
    //the semicolon (;) at the end of the next if removed by ksg on 10/21/04.
  Time new_t0 = (*ts)->getStartTime();
  Time expect_t0 = mLast_t0 + mIn.getStride();
  if (Debug()>0) cout << "new_t0=" << new_t0 << " last_t0=" << mLast_t0 
		      << " expect_t0=" << expect_t0 
		      << " Equal(?)=" << (new_t0==expect_t0) << endl;
  mLast_t0 = new_t0;

  /* If this time interval is more than one stride later than last, we have
     skipped some data and must wait before checking conditions */
  // the following line added by ksg 10/21/04 
  // cout << "at line 394 at  secs_per_rms loop by ksg new_t0= " << new_t0 
  //<< " expect_t0=" << expect_t0 
  //<< "mNconsecutive_last=" << mNconsecutive_last << " mNconsecutive=" <<
  // mNconsecutive << " ring_down=" << ring_down << endl;
  //1/22/05 KSG. In the following if statement mNconsecutive_last>0 has been changed
  // to mNconsecutive >0 in order ot be consistent
  if (new_t0>expect_t0 && mNconsecutive_last>0) {
    mNconsecutive = 1;
    //minute_gap is set equal to true if the current minute has a gap added KSG 11/05/04
    minute_gap = true;
    //Next line added 2/20/05 by KSG in order to make max_minutes_since =1
    // only 1 for ordinary data loss segments instead of ringdowns
    max_minutes_since=1;
    // The next line was added by KSG on 1/06/05 in order to calculate
    //gap_size_t0, the size of the gap in the minute trend
    gap_size_t0 = new_t0 - expect_t0;
    //Also on 1/06/05 KSG added the next line to zero the minutes_since gap counter in the case of gap
    minutes_since_gap = 0;
    // the following line was added 2/21/05 by ksg to set ring_down =false
    // since this branch of the if statement marks only ordinary data loss
    // not filter ring downs; then removed on 2/22/05
    //ring_down = false;
    //mLast_value = new_value;
    //printf("line 398 data loss in blrms seismic monitor found by ksg");
    cout << "data loss new_t0= " << new_t0 << " expect_t0= " << expect_t0 << " gap_size= " << gap_size_t0 <<endl;

      //" mNconsecutive = "<<  mNconsecutive <<endl;
  }
  else if (new_t0==expect_t0) {
    //The following line was a change made by KSG on 1/22/05 to fix
    // a problem with mNconsecutive_last (previously it only updated
    //once a minute
    //A line was added by ksg on 2/20/05 to set ring_down =false
    //and removed on 2/22/05
    mNconsecutive_last=mNconsecutive;
   mNconsecutive++;
   //ring_down=false;
    // The following two lines were added by KSG 1/17/05 to properly
    //increment the minutes_since_gap counter and zero the gap_size_t0 counter
    //minutes_since_gap++; this was a mistake (01/20/05 KSG)
    //The next line was also added by KSG on 1/17/05
    //gap_size_t0=0; this was also a mistake (KSG 1/20/05)

    // printf("no data loss line 403");
    // cout << "no data loss mNconsecutive=" << mNconsecutive << endl 
    // cout << "new_t0= " << new_t0 << "expect_t0= " << expect_t0 
    //<< "mNconsecutive = "<<  mNconsecutive <<endl;
  }
  //1/22/05 KSG. The following elseif branch was added to take care of the case where
  // we are considering the very first second of data, which is also the start of a filter ringdown
  else if ( new_t0 >expect_t0 && mNconsecutive_last==0) {
    mNconsecutive_last =1;
      mNconsecutive=2;
      ring_down=true;
      gap_size_t0=new_t0 - expect_t0;
	//The following line added by ksg on 2/20/05 in order
	// to mark ringdowns with 3 minute gaps rather than the 1 minute
	// gaps used for ordinary data loss episodes
	max_minutes_since = 3;}

  // End of time gap checking segment inserted by KSG on 9/30/04 from Keith Riles.
  // This is apparently the point in the program where the rms minute trends are writen out 10/29/04 KSG.
  if(segment_counter == rmspar->secs_per_rms_reading) {

    if( !first_reading_done ) {
      //cout << "line 414 print minute trends mNconsecutive =" << mNconsecutive << "segment_counter =" << segment_counter << endl;
      //cout << "rmspar=" << rmspar << endl;
      // The following printf uncommented by KSG temporarily on 10/29/04
      //printf("Writing segment. Column Tally is %d \n",column_tally);
      segment_counter = 0;
      //send current gps time to output file
      tnow = (*ts)->getStartTime();
      file_output->gps_time_for_entry = tnow.getS();
      //if there is an update file, open it
      
      // printf("Writing update file.\n");
      if(strcmp(rmspar->update_filename , "none") != 0) {
	uout = fopen(rmspar->update_filename,"w");
      }
      
      // printf("Sending data to output file.\n");
      //send data to output file
      mNcon_diff=mNconsecutive-mNconsecutive_last;
    // The following line was added by KSG on 1/20/05 to
    //increment the minutes_since_gap counter and zero the gap_size_t0 counter
    //in the following if statement mNconsecutive_last has been replaced by 
      //mNconsecutive. This was done by KSG on 1/22/05
      //The following line commented out and changed below by KSG 1/27/05
	// so that minutes_since_gap updating works after minute gaps
	// (data loss) incidents as well as ring down incidents
      //if(!minute_gap && mNconsecutive>1){minutes_since_gap++;}
      if(mNconsecutive > 1){minutes_since_gap++;}
     //cout << "just befor gap minute_gap=" << minute_gap << "minutes_since_gap
     //=" << minutes_since_gap << " max_minutes_since =" << max_minutes_since
     //<< " mNconsecutive_last="<< mNconsecutive_last <<endl;

      int bad_gap=0;
      int not_done=1;

      if( minutes_since_gap <= max_minutes_since){
       cout << " we found a gap minute_gap=" << minute_gap << " minutes_since_gap=" << minutes_since_gap << " max_minutes_since =" << max_minutes_since 
        << " ring_down =" << ring_down <<endl;

       bad_gap=1;
       not_done=2;
       was_gap=1;
      }
       
    //  cout<<".";
     while(not_done){
      not_done--;

      if(not_done!=0) cout<<"Extra Pass"<<endl;
      for(int m=0; m<column_tally; ++m) {
	// calculate rms from sum of squares
	// 11/01/04 added by KSG; If there is a data gap identified by
	//mNconsecutive being <segment_counter usually =60 
	//then set the elements of the data_strore
	//array equal to -1 (this is the array with the DMT data)
	//cout << "at line 440 where we fill the data_store array" << endl;
	//cout << "mNconsecutive= " << mNconsecutive << " segment_counter= " << segment_counter << endl;
	//if((mNconsecutive+1)%60 == 0)
	//if(mNcon_diff == 60)  {
	// The following if statement was changed by KSG on 1/6/05
	// and the original if statement is commented out
	//if( !minute_gap ) This if statement was used formerly KSG 1/20/05

/*	if( minutes_since_gap <= max_minutes_since){
	  //cout << " data loss or ring down with boolean ring_down =" << ring_down << endl;
	// printf("%f\n",(float)-sqrt(data_store[m]));
	  //KSG added on  2/20/05 the option to deal with the case of
	  //data loss or ring downs seperately using the boolean ring_down
	  if( ring_down ){
	    //first the case of a ring down marked with a 0
	    data_store[m] = (float)(0);
//add the rms values to the output file
	file_output->add_data((float)sqrt((double)data_store[m]),m);
          }
	  if(minute_gap) {
	    //Now the case of ordinary data loss marked with a -1 
	    data_store[m] = (float)(1);
//add the rms values to the output file
	file_output->add_data((float)-sqrt((double)data_store[m]),m);
	  } 
   }
*/

           if(bad_gap){
            //  if (not_done==1)
       	     //    data_store[m] = -1;
            //  else
		// data_store[m] = -1;

	        file_output->add_data(-1,m);
            }
           



	  // 02/21/05 KSG The following 2 lines no longer needed since
	  // we now seperately handle filter ringdowns and ordinary data loss
//add the rms values to the output file
//	file_output->add_data((float)-sqrt((double)data_store[m]),m);

//KSG 02/26/05 The bracket below ends the if(minutes_since_gap > max_minutes_since) statement above (inside the for loop above)
  // if(m==3) cout<<"Reset Data first value: "<<data_store[0]<<endl;
// KSG This is for the case of no data loss incident: The case of normal data 
	 else { 
	data_store[m] /= (float)( (rmspar->secs_per_rms_reading)*
				  (file_output->samp_rate_array[m]) );
	// printf("%f\n",(float)sqrt(data_store[m]));
// add the rms values to the output file
	file_output->add_data((float)sqrt((double)data_store[m]),m);
// KSG 02/25/05 this ends the else statement just 5 lines above
	} 
	//printf("%f\n",(float)sqrt(data_store[m]));

//***Check Here for time error!!!!!
	// if trend writing is enabled, add data point to trend
     // if(m==3)
     //   cout<<"Data Store 2nd Value :"<<data_store[0]<<endl;

	if( make_trends ) {
	 pblrmsTrend->trendData
	    ( (trend_channel_list[m]).c_str(),
	      tnow,
	      (double)data_store[m] );
	  // cout << "adding data to channel "
	  //     << trend_channel_list[m].c_str() << endl;}
	// end the if(make_trends... statement 6 lines above ksg 2/25/05
	}


	// end of adding data point to trend
	// The statement below was commented out by KSG on 01/22/05
	//becuase mNconsecutive_last should be incremented every second
	// not every minute. And an appropriate correction has been added above
	//mNconsecutive_last=mNconsecutive;
	// added 11/05/04 by KSG zero minute_gap the boolean variable 
	//that equals true if a gap was found in the past minute
	//minute_gap = false;
	//cout << "at line 443 10/04 at timeseries mod 10/04" << endl;
	// this section modifies the timeseries containing dtt data and
	// sends the timeseries to the dtt.
	(*(monitor_tseries + m))->getData((size_t)monitor_length,
					  monitor_init);
//***Check Here for time error!!!!!
	// update the time series
	for(int p=0 ; p<(monitor_length - 1) ; ++p) {
	  monitor_init[p] = monitor_init[p+1];
	}

	if(!bad_gap) monitor_init[monitor_length - 1] = (float)sqrt((double)data_store[m]);
	else monitor_init[monitor_length-1]=-1;
    
      //  cout<<"Sqrted:  "<< (float)sqrt((double)data_store[m])<<" "<<monitor_init[monitor_length - 1]<<" "<<monitor_init[monitor_length - 2]<<"   k"<<(trend_channel_list[m]).c_str()<<" "<<bad_gap<<endl;
                        
 

	(*(monitor_tseries + m))->setData
	  (tnow - (*(monitor_tseries+m))->getInterval() ,
	   (*(monitor_tseries+m))->getTStep(),
	   monitor_init,
	   (size_t)monitor_length );

	// send latest data point from channel to updated channel list
	// printf("Writing to update file.\n");

	if(strcmp(rmspar->update_filename,"none") != 0 && !bad_gap) {
	  fprintf(uout,"%s %e\n",
		  monitor_names + m*max_monitor_name_length,
		  (float)sqrt((double)data_store[m]) );
	}

	//**I think this is the end of the for loop with index m that**
	//** prints out the seismic channels one (column) at a time 01/17/05 KSG**
      }}
      //here we set minute_gap=false KSG 2/27/05 
      //and we set ring_down=false for minutes when
      //the time_since gap exceeds max_minutes_since, which means
      //the data gap or ring down has timed out
	// We fixed the minutes_since_gap updating on 2/27/05 above
	// about line 491 so the if statement below should work for minute_gaps now
	if( minutes_since_gap > max_minutes_since){
      minute_gap= false;
      ring_down= false;
      }
      //reset data_store
      for(int p=0;p<column_tally;++p) {
	data_store[p] = 0;
       if (p==0)cout<<"Reset Data Store: "<<data_store[0]<<endl;
      }
      
      //if open, close update file
      if(strcmp(rmspar->update_filename,"none") != 0) {
	fclose(uout);
      }
      
    } else { 
      first_reading_done = true;
    }
  }
  
  
  //---------------------------------------- seconds----------------
  
  // printf("Got data set %d. Column tally is %d\n",segment_counter,
  //	 column_tally);

  if ( second_counter == total_seconds_to_run ) {
    finish();
  }

  return;
}

























